all:V: opt byte $CAML/validate.out



<$CAML/ocaml.mk
<|$CAML/cube -rules

$CAML/validate.out:D: $CAML/validate $CAML/interps.nw
	$TOP/in-dir-of $CAML/validate ./validate > $target

boot:V:
	mk src
	mk depend

clobber:V: clobber-caml

clean:V: clean-caml
clean-caml:V:
	rm -f $CAML/*~

LINE =          '# %L "%F"%N'

NOWEAVE =       noweave
NOTANGLE =      notangle
CPIF =          cpif

LATEX =         latex
PS2PDF=         ps2pdf
RERUN =         Rerun (LaTeX|to get cross-references right)

NOCOND =        ./nocond $INTERP
OCAMLDEFS =     ./autodefs.ocaml

#
# general rules to extract code from noweb files
#


# ------------------------------------------------------------------ 
# uLua scanner and parser. We need some extra rules to extract sources
# from noweb files. 

$CAML/parsex.mly: $CAML/syntax.nw
	[ -r $target ] && chmod +w $target
	notangle -Rparser.mly $prereq | 
	sed "s/','/COMMA/g; \
       s/'{'/LBRA/g;  \
       s/'}'/RBRA/g;  \
       s/'<'/LT/g;    \
       s/'>'/GT/g;    \
       s/':'/COLON/g; \
       s/';'/SEMI/g;  \
       s/'\\^'/HAT/g; \
       s/'\\*'/STAR/g;\
       s@'/'@SLASH@g; \
       s/'('/LPAR/g;  \
       s/')'/RPAR/g;  \
       s/'='/GETS/g;  \
       s/'+'/PLUS/g;  \
       s/'-'/MINUS/g; \
       s/'\\.'/DOT/g; \
       s/'\\['/LSQ/g; \
       s/'\\]'/RSQ/g" > $target
	chmod -w $target

$CAML/scanner.mll: $CAML/syntax.nw
	notangle -R`basename $target` $prereq > $target

$CAML/luafloat.mll: $CAML/syntax.nw
	notangle -R`basename $target` $prereq > $target

# generate a self-contained LaTeX document from a LaTeX file that
# is supposed to be read by \input from a master LaTeX file. This rule
# allows to specify extra packages like tabularx to be included. 
# The here-document ended by EOF relies on a single TAB character at
# the beginning of a line - so don't substitute spaces for it .

NOWEBBREAKCODE=no

%.tex:Q: %.inc
	(tr '\n' ' ' <<EOF
	\documentclass{article}
	\usepackage{noweb}
	\usepackage{tabularx}
	\usepackage{hyperref}
	\usepackage{path}
	\usepackage[dvips]{graphicx}
	\pagestyle{noweb}
	`if [ $NOWEBBREAKCODE = yes ]; then echo '\noweboptions{breakcode}'; fi`
	\begin{document}
	\nwfilename{$stem.nw}
	\tableofcontents
	\nwbegindocs{}
	EOF
	cat $stem.inc
	tr '\n' ' ' <<EOF
	\bibliographystyle{alpha}
	\bibliography{qc--}
	\end{document}
	EOF
	) > $target

%.inc:          %.nw
                $NOWEAVE -delay -filter $OCAMLDEFS -index $prereq > $target

%.html: %.nw
        cat $TOP/config/macros.tex $prereq |\
        $NOWEAVE -delay -filter $OCAMLDEFS -index -html -filter l2h - > $target

%.bbl:  %.bib %.aux
        bibtex $stem 
                    
%.dvi:  %.tex 
        $LATEX "\\scrollmode \\input $stem.tex"
        ltxcount=3
        while egrep -s "$RERUN" $stem.log && [ $ltxcount -gt 0 ]
        do
            $LATEX "\\scrollmode \\input $stem.tex"
            ltxcount=`expr $ltxcount - 1`
        done

%.ps:   %.dvi
        dvips -Ppdf -o $target $prereq


%.pdf:  %.ps
        ps2pdf $prereq


%.dep: %
	ocamldep -I `dirname $prereq` $prereq > $target
